Package: mini007
Type: Package
Title: Lightweight Framework for Orchestrating Multi-Agent Large
        Language Models
Version: 0.4.0
Authors@R: 
    person(given = c("Mohamed", "El", "Fodil"),
           family = "Ihaddaden",
           role = c("aut", "cre"),
           email = "ihaddaden.fodeil@gmail.com")
Description: Provides tools for creating agents with persistent state using 
    R6 classes <https://cran.r-project.org/package=R6> and the 'ellmer' 
    package <https://cran.r-project.org/package=ellmer>. Tracks prompts, 
    messages, and agent metadata for reproducible, multi-turn large language 
    model sessions.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/feddelegrand7/mini007
BugReports: https://github.com/feddelegrand7/mini007/issues
Imports: checkmate (>= 2.3.1), cli (>= 3.6.5), DiagrammeR (>= 1.0.11),
        ellmer (>= 0.4.0), glue (>= 1.8.0), R6 (>= 2.6.1), rlang (>=
        1.1.6), uuid (>= 1.2.0)
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-27 13:26:58 UTC; mohamedelfodilihaddaden
Author: Mohamed El Fodil Ihaddaden [aut, cre]
Maintainer: Mohamed El Fodil Ihaddaden <ihaddaden.fodeil@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-27 13:50:02 UTC
Built: R 4.5.2; ; 2026-05-27 16:23:02 UTC; unix
